#include "okindd.h"

service::service(QApplication *application)	: QDBusAbstractAdaptor(application){
	resetToDefault();

	QDBusInterface runningExit ("org.kochkin.okindd", "/Service");
	runningExit.call ("exit");
	sleep (1);

	QDBusConnection dbusConnection = QDBusConnection (QDBusConnection::sessionBus());
	dbusConnection.connectToBus(QDBusConnection::SessionBus, "org.kochkin.okindd");
	if (!dbusConnection.registerService("org.kochkin.okindd")) {
		qDebug ("Service not registered. May be already running");
		}
	if (!dbusConnection.registerObject("/Service", this, QDBusConnection::ExportAllSlots)) {
		qDebug ("Object /Service not registered.");
		}
	workValue=defValue;
	roundInd = new roundIndicator(0);
	pict = new pic (0);
	text = new textObject (0);

	setConfigFile ("");
}

service::~service(){}

// Установка значений профилей по умолчанию
void service::resetToDefault(){
	HomeDir=home.homePath();
	defValue["Picture"]="false";
	defValue["PictureLeft"]="center";
	defValue["PictureTop"]="center";
//	defValue["PictureFile"]="/usr/share/okindd/din.png";
	defValue["PictureFile"]="/usr/share/icons/oxygen/128x128/actions/application-exit.png";
	defValue["PictureTimer"]="1000";
	defValue["PictureOpacity"]="0.8";
	defValue["PictureOffset"]="10";

	defValue["RoundIndicator"]="false";
	defValue["RoundIndicatorTimer"]="1000";
	defValue["RoundIndicatorLeft"]="center";
	defValue["RoundIndicatorTop"]="center";
	defValue["RoundIndicatorDiameter"]="300";
	defValue["RoundIndicatorAngle"]="300";
	defValue["RoundIndicatorPicture"]="/usr/share/icons/oxygen/128x128/actions/application-exit.png";
	defValue["RoundIndicatorShadow"]="";
	defValue["RoundIndicatorColorULB"]="50,100,120,255";
	defValue["RoundIndicatorColorBRB"]="0,5,10,255";
	defValue["RoundIndicatorColorULI"]="255,255,255,255";
	defValue["RoundIndicatorColorBRI"]="0,20,30,255";
	defValue["RoundIndicatorColorULC"]="0,0,0,255";
	defValue["RoundIndicatorColorBRC"]="0,80,90,255";
	defValue["RoundIndicatorValueMin"]="0";
	defValue["RoundIndicatorValueMax"]="100";
	defValue["RoundIndicatorDigitsEnabled"]="true";
	defValue["RoundIndicatorDigitsColor"]="200,200,200,150";
	defValue["RoundIndicatorDigitsColorShadow"]="0,0,0,150";
	defValue["RoundIndicatorDigitsFont"]="Liberation sans";
	defValue["RoundIndicatorDigitsFontBold"]="true";
	defValue["RoundIndicatorDigitsFontItalic"]="false";
	defValue["RoundIndicatorOpacity"]="0.8";
	defValue["RoundIndicatorCenterDiameter"]="Described";
	defValue["RoundIndicatorTimer"]="1000";
	defValue["RoundIndicatorValueFromDBUS"]="";
	defValue["RoundIndicatorOffset"]="50";

	defValue["RemoteIncrease"]="";
	defValue["RemoteDecrease"]="";
	defValue["RemoteIndicatorProfile"]="";

	defValue["TextBoxLeft"]="center";
	defValue["TextBoxTop"]="center";
	defValue["TextBoxStyle"]="background-color:gray; border:2px solid black; font-size:50px; color:black;";
	defValue["TextBoxOpacity"]="0.8";
	defValue["TextBoxBorderSize"]="50";
	defValue["TextBoxTransparent"]="false";
	defValue["TextBoxOffset"]="50";
	defValue["TextBoxShadow"]="";
}

// Загрузка профиля
void service::getProfiles(){
		QSettings cfg (configFile, QSettings::NativeFormat);
		QStringList profiles = cfg.childGroups();
}

// Выход
void service::exit(){
	qApp->quit();
}

// Запуск профиля с обработкой OSDoff
void service::display (QString profile, int value){
	if (profile != "OSDoff") fdisplay (profile,value);
	else roundInd->hide();
}

// Запуск профиля без обработки OSDoff
void service::fdisplay (QString profile, int value){
	loadProfileFromArray (profile);
// Если включен прогресс-круг -> установка его параметров
	if (strToBool (workValue["RoundIndicator"])) roundInd->showCurrent (value);
// Если включена картинка -> показ
	if (strToBool (workValue["Picture"])) pict->showCurrent();
}

// Изменение текущего профиля
void service::profileChanged(){
//	workValue=configValue;
}

// Выбор файла конфигурации и загрузка массива из файла конфигурации
void service::setConfigFile (QString confFile){
	configFile=confFile;
	if (confFile=="") configFile=HomeDir+"/.okindd/okinddrc";
	QSettings cfg (configFile, QSettings::NativeFormat);
	QStringList groups = cfg.childGroups();
	profiles.clear();
	OkProfile profile;
	for (int i=0;i<groups.size();i++){
		profile.name=groups.at(i);
		cfg.beginGroup(profile.name);
		profile.parameters=cfg.childKeys();
		profile.values.clear();
		for (int j=0;j<profile.parameters.size();j++){
			profile.values.append (cfg.value (profile.parameters[j], "").toString());
			}
		cfg.endGroup();
		profiles.append (profile);
		}
}

// Получение параметров профиля из массива
void service::loadProfileFromArray (QString pr){
	currentProfile=pr;
	setConfigValue (pr,"Picture");
	if (strToBool (workValue["Picture"])){
		setConfigValue (pr,"PictureLeft");
		setConfigValue (pr,"PictureTop");
		setConfigValue (pr,"PictureFile");
		setConfigValue (pr,"PictureTimer");
		setConfigValue (pr,"PictureOpacity");
		setConfigValue (pr,"PictureOffset");
		pict->picture_pic->load (workValue["PictureFile"]);
		}
	setConfigValue (pr,"RoundIndicator");
	if (strToBool (workValue["RoundIndicator"])){
		setConfigValue (pr,"RoundIndicatorTimer");
		setConfigValue (pr,"RoundIndicatorLeft");
		setConfigValue (pr,"RoundIndicatorTop");
		setConfigValue (pr,"RoundIndicatorDiameter");
		setConfigValue (pr,"RoundIndicatorAngle");
		setConfigValue (pr,"RoundIndicatorPicture");
		setConfigValue (pr,"RoundIndicatorShadow");
		setConfigValue (pr,"RoundIndicatorColorULB");
		setConfigValue (pr,"RoundIndicatorColorBRB");
		setConfigValue (pr,"RoundIndicatorColorULI");
		setConfigValue (pr,"RoundIndicatorColorBRI");
		setConfigValue (pr,"RoundIndicatorColorULC");
		setConfigValue (pr,"RoundIndicatorColorBRC");
		setConfigValue (pr,"RoundIndicatorValueMin");
		setConfigValue (pr,"RoundIndicatorValueMax");
		setConfigValue (pr,"RoundIndicatorDigitsEnabled");
		setConfigValue (pr,"RoundIndicatorDigitsColor");
		setConfigValue (pr,"RoundIndicatorDigitsColorShadow");
		setConfigValue (pr,"RoundIndicatorDigitsFont");
		setConfigValue (pr,"RoundIndicatorDigitsFontItalic");
		setConfigValue (pr,"RoundIndicatorDigitsFontBold");
		setConfigValue (pr,"RoundIndicatorOpacity");
		setConfigValue (pr,"RoundIndicatorCenterDiameter");
		setConfigValue (pr,"RoundIndicatorValueFromDBUS");
		setConfigValue (pr,"RoundIndicatorOffset");
		roundInd->picture.load (workValue["RoundIndicatorPicture"]);
		roundInd->shadow.load (workValue["RoundIndicatorShadow"]);
		}
	setConfigValue (pr,"RemoteIncrease");
	setConfigValue (pr,"RemoteDecrease");
	setConfigValue (pr,"RemoteIndicatorProfile");
}

QString service::getCurrentConfigFile(){
	return configFile;
}

QString service::getCurrentProfile(){
	return currentProfile;
}

QString service::getVersion(){
	return VERSION;
}

void service::remote (QString profile, QString action){
	loadProfileFromArray (profile);
	if (workValue["RemoteIncrease"] != "" && workValue["RemoteDecrease"] != "" && workValue["RemoteIndicatorProfile"] != ""){
		if (action.toUpper() == "INCREASE"){
			QString command=workValue["RemoteIncrease"];
			QList<QVariant> vparam;
			QStringList param=command.section (" ",3,10).split (" ");
			foreach(QString s, param) vparam << s;
			QDBusInterface i (command.section (" ",0,0), command.section (" ",1,1));
			i.callWithArgumentList (QDBus::NoBlock, command.section (" ",2,2), vparam);
		}
		if (action.toUpper() == "DECREASE"){
			QString command=workValue["RemoteDecrease"];
			QList<QVariant> vparam;
			QStringList param=command.section (" ",3,10).split (" ");
			foreach(QString s, param) vparam << s;
			QDBusInterface i (command.section (" ",0,0), command.section (" ",1,1));
			i.callWithArgumentList (QDBus::NoBlock, command.section (" ",2,2), vparam);
		}
		fdisplay (workValue["RemoteIndicatorProfile"], 0);
	}
}

QString service::getCurrentDateTime (QString format){
	QDateTime ct=QDateTime::currentDateTime();
	return ct.toString (format);
}
